# manually fit an annual variogram using observations and the UK residuals

# inputs:
#   df_obs_sub
#   local_stations


library(dplyr)
library(gstat)

# PARAMETERS ----
input_path <- "/esarchive/scratch/ccarnere/RScripts/"
pollutant <- "o3"
year <- 2022

crs_latlon <- "+proj=longlat +datum=WGS84 +ellps=WGS84 +towgs84=0,0,0"
crs_utm <- "+proj=utm +zone=31 +ellps=intl +units=m +no_defs"

# ---


input_path <- (paste0(input_path,"results_regional_",year,"/results_gene_",
                      pollutant,"/output/"))


# (obs) variogram with annual means ----

# inputs
df_obs_sub <- readRDS(paste0(input_path,"df_obs_gene_",pollutant,".rds"))
local_stations <- readRDS(paste0(input_path,"local_stations_gene_",pollutant,".rds"))

# annual average of observations
obs_annual_mean <- df_obs_sub %>%
  dplyr::summarise(across(where(is.numeric), ~ mean(.x, na.rm = TRUE)))

# transpose 
obs_annual_mean_t <- data.frame(ref_obs = colnames(obs_annual_mean),
                                obs = as.numeric(obs_annual_mean[1,]))


stations <- data.frame(lat=local_stations@coords[,2],
                       lon=local_stations@coords[,1],
                       ref_obs=local_stations@data$ref_obs) 


df <- merge(stations, obs_annual_mean_t, by="ref_obs")
df <- na.omit(df) 
# df has contains station_name, lat, lon, obs


coordinates(df)     <- ~lon+lat
sp::proj4string(df) <- crs_latlon

#config the coordinates in UTM
sp <- spTransform(df, sp::CRS(crs_utm))

# Compute variogram
variogram <- automap::autofitVariogram(formula=obs~1,
                                       input_data=df, 
                                       verbose=FALSE)

saveRDS(variogram, paste0(input_path,"avg_variogram_obs_",pollutant,".RDS"))
saveRDS(sp, paste0(input_path,"avg_sp_variogram_obs_",pollutant,".RDS"))



#... modify+plot variogram ----

# manually modify the variogram
# we want a small nugget but >0, and a meaningful range and sill
variogram <- automap::autofitVariogram(formula=obs~1,input_data=sp,
                                       verbose=FALSE, model="Gau",
                                       fix.values=c(20,20000,150))  #c(nugget,range,sill)
# plot(variogram)

# gstat also has an alternative function to fit variograms
# df_gstat <- gstat(formula=obs~1, data = sp)
variogram_g <- gstat::variogram(obs~1, sp, map=TRUE, cutoff=50000, width=1000)
# plot(variogram_g)



#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-
#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-


# (UK) variogram with annual means ----

format <- ifelse(pollutant %in% c("so2","co"),"ghost","gene")

# inputs
# LOOCV_total_df <- readRDS(paste0(input_path,"LOOCV_gene_",pollutant,"_df.rds"))
# model_raw_at_AQMS_df <- readRDS(paste0(input_path,"/model_raw_at_AQMS_",format,"_",pollutant,"_df.rds"))

mod_obs_data <- readRDS(paste0("/esarchive/scratch/ccarnere/AQS_database/postprocessed/regional/statistics/",
                               ifelse(!pollutant %in% c("pm10","pm2p5"),
                                      paste0("sconc",pollutant,"/"),
                                      paste0(pollutant,"/")),
                               "mod_obs_stations_",pollutant,"_",year,".RDS"))

df_uk <- mod_obs_data %>%
  filter(model=="raw_model") %>%
  select(-model)

# annual mean
df_uk <- df_uk %>%
  group_by(station) %>%
  summarise(obs = mean(obs, na.rm=T),
            mod = mean(mod, na.rm=T))

# calculate bias
df_uk <- df_uk %>%
  mutate(bias = mod-obs)

# add lat,lon and transform to UTM coords (use dataframe stations)
df_uk <- merge(stations, df_uk, by.x="ref_obs", by.y="station")
df_uk <- na.omit(df_uk) 
# df_uk has contains station_name, lat, lon, obs

sp_uk <- df_uk
coordinates(sp_uk)     <- ~lon+lat
sp::proj4string(sp_uk) <- crs_latlon

#config the coordinates in UTM
sp_uk <- spTransform(sp_uk, sp::CRS(crs_utm))



# ??? hauria de ser obs~bias???
variogram_uk <- automap::autofitVariogram(formula=obs~mod,
                                          input_data=sp_uk,
                                          verbose=FALSE)

saveRDS(variogram_uk, paste0(input_path,"avg_variogram_uk_",pollutant,".RDS"))
saveRDS(sp_uk, paste0(input_path,"avg_sp_variogram_uk_",pollutant,".RDS"))

#... modify+plot variogram ----

# manually modify the variogram
# we want a small nugget but >0, and a meaningful range and sill
variogram <- automap::autofitVariogram(formula=obs~mod,input_data=sp,
                                       verbose=FALSE, model="Gau",
                                       fix.values=c(20,20000,150))  #c(nugget,range,sill)
# plot(variogram)
