# script to transform from an .rds file containing one day of the regional correction 
# to daily .nc following conventions
# 2023-10-05
# Cristina Carnerero
# BSC-ES AQS

library(dplyr)
library(ncdf4)
library(lubridate)


# PARAMETERS ---
# pol_list = c("no2","o3","pm10","pm2p5","co","so2")
# pol_list = c("so2","co")
# pol_list = c("pm2p5")
# gbm = FALSE
# year_list = c(2017,2019:2023)
# ---

args = commandArgs(TRUE)
if(length(args) %in% c(3,4)) {
  pol <- args[1]
  year <- args[2]
  save_result <- args[3]
  if(length(args)==4 & tolower(args[4])=="gbm"){
    with_gbm <- TRUE
  }else{
    with_gbm <- FALSE
  }
} else {
  #cat("\n***Arguments missing***\n Arguments: pollutant year \n Pollutant codes: no2, o3, pm10, pm2p5, so2, co")
  stop("***Arguments missing*** Arguments: pollutant year save_result. Pollutant codes: no2, o3, pm10, pm2p5, so2, co\n Optionally 3rd argument: gbm")
}

print(paste0("Pollutant: ",pol,
             ", Year: ",year,
             ", GBM: ",with_gbm))



# for(pol in pol_list){
# print(pol)

# variable name mapping
if(pol=="o3"){
  var_name <- "sconco3"
  var_long_name <- "Ozone Concentration"
  res <- "hourly"
}else if(pol=="no2"){
  var_name <- 'sconcno2'
  var_long_name <- "Nitrogen Dioxide Concentration"
  res <- "hourly"
}else if(pol=="pm10"){
  var_name <- 'pm10'
  var_long_name <- "PM10 Concentration"
  res <- "daily_means"
}else if(pol=="pm2p5"){
  var_name <- 'pm2p5'
  var_long_name <- "PM2.5 Concentration"
  res <- "daily_means"
}else if(pol=="so2"){
  var_name <- "sconcso2"
  var_long_name <- "Sulphur Dioxide Concentration"
  res <- "hourly"
}else if(pol=="co"){
  var_name <- "sconcco"
  var_long_name <- "Carbon Monoxide Concentration"
  res <- "hourly"
}else if(pol=="benzene"){
  var_name <- "benzene"
  var_long_name <- "Benzene Concentration"
  res <- "hourly"
}


# where results will be saved
output_path <- paste0(save_result, "netCDF/")

if (!dir.exists(output_path)) {
  dir.create(output_path, recursive = TRUE)
}

# make sure that the output directory exists
dir.create(file.path(output_path))

# for(year in year_list){
  
# reference simulation to extract the coordinates, it can be a different variable
if(as.numeric(year)==2018){
  ref_path = "/esarchive/exp/wrf-hermes-cmaq/b064/ip/hourly/sconcno2/sconcno2_2018040600.nc"
}else{
  ref_path = "/esarchive/exp/auto-wrf-hermes-cmaq/a52i/cat/hourly/sconcno2/sconcno2_2023010100.nc"
  
}


# extract coordinates from reference nc
ref_nc <- nc_open(ref_path)

lat_model <- ncvar_get(ref_nc, 'lat')
lon_model <- ncvar_get(ref_nc, 'lon')

x_model <- ncvar_get(ref_nc, 'x')
y_model <- ncvar_get(ref_nc, 'y')

x_model_att <- ncatt_get(ref_nc, 'x')
y_model_att <- ncatt_get(ref_nc, 'y')

lambert <- ncatt_get(ref_nc, "Lambert_conformal")
lev <- ncatt_get(ref_nc, "lev")

rm(ref_nc)

#input_path = paste0("/esarchive/scratch/ccarnere/RScripts/results_regional_",year,"/results_gene_",pol,"/output")
input_path <- save_result


# static variables
var_units <- "ug/m3"
proj <- "+proj=longlat +datum=WGS84 +no_defs"

xdim <- ncdim_def("x", "1000 m", x_model, longname = "x coordinate of projection")
ydim <- ncdim_def("y", "1000 m", y_model, longname = "y coordinate of projection")


# create a manual sequence of dates
dates <- seq(from=as.POSIXct(paste0(year,"-01-01 00:00"), tz="UTC"),
             to=as.POSIXct(paste0(year,"-12-31 23:00"), tz="UTC"),
             by="hour")  

days <- unique(date(dates))

# transform the df and save one nc file per day
for(i in 1:length(days)){
  # i<-1 
  # extract the day and create the output filename
  day_i <- days[i]
  date_fmt <- format(day_i, "%Y%m%d")
  print(date_fmt)
  out_file <- paste0(output_path,var_name,"_",date_fmt,"00.nc")
  
  # read individual daily file
  if(with_gbm){
    gbm_suffix <- "gbm_"
  }else{
    gbm_suffix <- ""
  }
  
  if(year>=2022){
    input_file <- paste0(input_path, "/UK_gene_",pol,"_",gbm_suffix,date_fmt,".rds")
  }else if(year=="2018" | pol=="so2" | pol=="co" ){
    input_file <- paste0(input_path, "/UK_ghost_",pol,"_",gbm_suffix,date_fmt,".rds")
  }else{
    input_file <- paste0(input_path, "/UK_gene_",pol,"_",gbm_suffix,date_fmt,".rds")
  }
  
  if(file.exists(input_file)){
      df_input <- readRDS(input_file)
  }else{
    print("file not found")
    next
  }

  # if df_input only has 1 column, that means that it is a daily file (PM)
  if(length(df_input)==1){
    dates <- days
  }

  # subset dates for corresponding day
  dates_i <- dates[date(dates)==day_i]
  
  for(j in 1:length(df_input)){
    dim(df_input[[j]]) <- c(lat=length(lat_model[1,]),
                            lon=length(lon_model[,1]))
  }
  
  result <- array(0, dim=c(length(lat_model[1,]),length(lon_model[,1]),length(dates_i)))
  
  for (l in 1:length(dates_i)) {
    result[,,l] <- as.matrix(df_input[[l]]) 
  }
  
  
  
  # define nc variables
  index <- array(1:length(dates_i), c(pos_index = length(dates_i))) #pos_index???
  
  timedim <- ncdim_def("time", 
                       paste("hours since", as.Date(dates_i[1]), "00:00:00"),
                       index - 1)
  
  
  levdim <- ncdim_def("lev", units="", as.numeric(1))
  
  # var_def <- ncvar_def(var_name, var_units, list(ydim, xdim, timedim),
  #                      longname = var_long_name)
  var_def <- ncvar_def(var_name, var_units, list(xdim, ydim, levdim, timedim),
                       longname = var_long_name)
  
  lat_def <- ncvar_def("lat", "degrees_north", list(xdim, ydim), longname="latitude coordinate")
  lon_def <- ncvar_def("lon", "degrees_east", list(xdim, ydim), longname="longitude coordinate")
  
  lambert_def <- ncvar_def("Lambert_conformal", units="", dim=list())
  # lev_def <- ncvar_def("lev", units="", dim=list())
  
  # proj_def <- ncvar_def("WGS84", units="", dim=list(), longname="WGS84")
  
  # var_list <- list(var_def, lat_def, lon_def, proj_def)
  # var_list <- list(var_def, lat_def, lon_def, lev_def, lambert_def)
  var_list <- list(var_def, lat_def, lon_def, lambert_def)
  
  # save nc
  # ncout <- nc_create(out_file, vars = var_def, force_v4 = FALSE)
  ncout <- nc_create(out_file, vars = var_list, verbose=F, force_v4=T)
  
  
  ncvar_put(ncout, lat_def, lat_model)
  ncvar_put(ncout, lon_def, lon_model)
  
  ncvar_put(ncout, var_def, as.matrix(result))
  # ncatt_put(ncout, var_name, "grid_mapping", "crs")
  ncatt_put(ncout, var_name, "grid_mapping", "Lambert_conformal")
  ncatt_put(ncout, var_name, "coordinates", "lat lon")
  # ncatt_put(ncout, var_name, "proj4", proj)
  
  ncatt_put(ncout,"x","standard_name","projection_x_coordinate")
  ncatt_put(ncout,"y","standard_name","projection_y_coordinate")
  
  ncatt_put(ncout, "time", "calendar", "gregorian")
  ncatt_put(ncout, "time", "standard_name", "time")
  
  ncatt_put(ncout, 0, "date", as.character(Sys.time()))
  
  ncvar_put(ncout, lambert_def, as.integer(0))
  ncatt_put(ncout, "Lambert_conformal", "grid_mapping_name", "lambert_conformal_conic")
  ncatt_put(ncout, "Lambert_conformal", "standard_parallel", "43., 37.")
  # ncatt_put(ncout, "Lambert_conformal", "standard_parallel", "37., 43.")
  ncatt_put(ncout, "Lambert_conformal", "longitude_of_central_meridian", "-3")
  ncatt_put(ncout, "Lambert_conformal", "latitude_of_projection_origin", "40")
  
  # ncvar_put(ncout, lev_def, 0)
  ncatt_put(ncout, "lev", "positive", "up")
  
  # ncvar_put(ncout, proj_def, proj)
  # ncatt_put(ncout, "WGS84", "grid_mapping_name", "WGS84")
  
  nc_close(ncout)
}


print("**netcdf files saved**")