# find statistics files for all pollutants and make a plot showing statistics (MB, R, RMSE...)

library(tidyverse)
library(dplyr)

input_path <- "/esarchive/scratch/ccarnere/AQS_database/postprocessed/regional/statistics/"
output_path <- input_path
setwd(input_path)

statistic.list <- c("RMSE","r","R2","MB","COE")


# function to merge all files in a same directory containing a string
# this will include gbm if gbm files exist in the same path
merge_stats <- function(input_path, pol){
  df <- list.files(path = input_path,
                 pattern = paste0("Stats_",pol), 
                 recursive=TRUE) %>%
    map_dfr(readRDS)
  
  return(df)
}


# create a dataframe containing statistics for all pollutants and years --------
df_no2 <- merge_stats(input_path, "no2")
df_o3 <- merge_stats(input_path, "o3")
df_pm10 <- merge_stats(input_path, "pm10")  #this includes gbm as well
df_pm2p5 <- merge_stats(input_path, "pm2p5")  #this includes gbm as well
df_so2 <- merge_stats(input_path, "so2")
df_co <- merge_stats(input_path, "co")

df <- bind_rows(df_no2,df_o3,df_pm10,df_pm2p5,df_so2,df_co) %>%
  dplyr::select(pol,year,aggreg,model,n,any_of(statistic.list))

df_main <- df %>%
  dplyr::filter(!pol %in% c("so2","co"))

df_so2_co <- df %>%
  dplyr::filter(pol %in% c("so2","co"))



#TODO plot_model_vs_obs.R

# plot timeseries of R, MB, RMSE, COE for each pollutant -----------

text_size=15

df2 <- df %>% 
  dplyr::filter(aggreg=="hourly") %>%
  dplyr::mutate(R2 = r^2) %>%
  dplyr::select(-n, -aggreg) %>%
  dplyr::mutate(pol=toupper(pol)) %>%
  pivot_wider(names_from = c("model"), values_from = statistic.list)


for(statistic in statistic.list){
  dodge <- 0.7
  p <- df2 %>%
    dplyr::mutate(year=as.factor(year)) %>%
    ggplot(aes(x=year, 
               ymin=!!sym(paste0(statistic,"_raw_model")), 
               ymax=!!sym(paste0(statistic,"_UK_model")), 
               color=pol, group=pol)) +
    # geom_linerange(position = position_dodge(width = dodge), size = 5)+
    geom_point(aes(y=!!sym(paste0(statistic,"_raw_model"))),shape=21,position=position_dodge(width=dodge))+
    geom_point(aes(y=!!sym(paste0(statistic,"_UK_model"))),shape=20,size=4,position=position_dodge(width=dodge))+
    geom_linerange(aes(x=year,y=!!sym(paste0(statistic,"_raw_model"))),
                   color="black", position = position_dodge(width = dodge))+
    labs(y=statistic,
         x="",
         color="Pollutant") +
    scale_color_viridis_d(alpha=1)+
    theme_classic()+
    theme(text = element_text(size=text_size),
          panel.grid.major = element_blank(), 
          panel.background = element_rect(colour = "black", size=0.5),
          axis.text.x = element_text(angle = 45, hjust = 1))
  
  ggsave(paste0(output_path,statistic,".png"),
         width=9,
         height=6,
         units="in",
         dpi = 300)
}



## plot timeseries of R, MB, RMSE, COE for the MAIN pollutants -----------

df2 <- df_main %>% 
  dplyr::filter(aggreg=="hourly") %>%
  dplyr::mutate(R2 = r^2) %>%
  dplyr::select(-n, -aggreg) %>%
  dplyr::mutate(pol=toupper(pol)) %>%
  pivot_wider(names_from = c("model"), values_from = statistic.list)


for(statistic in statistic.list){
  dodge <- 0.7
  p <- df2 %>%
    dplyr::mutate(year=as.factor(year)) %>%
    ggplot(aes(x=year, 
               ymin=!!sym(paste0(statistic,"_raw_model")), 
               ymax=!!sym(paste0(statistic,"_UK_model")), 
               color=pol, group=pol)) +
    # geom_linerange(position = position_dodge(width = dodge), size = 5)+
    geom_point(aes(y=!!sym(paste0(statistic,"_raw_model"))),shape=21,position=position_dodge(width=dodge))+
    geom_point(aes(y=!!sym(paste0(statistic,"_UK_model"))),shape=20,size=4,position=position_dodge(width=dodge))+
    geom_linerange(aes(x=year,y=!!sym(paste0(statistic,"_raw_model"))),
                   color="black", position = position_dodge(width = dodge))+
    labs(y=statistic,
         x="",
         color="Pollutant") +
    scale_color_viridis_d(alpha=1)+
    theme_classic()+
    theme(text = element_text(size=text_size),
          panel.grid.major = element_blank(), 
          panel.background = element_rect(colour = "black", size=0.5),
          axis.text.x = element_text(angle = 45, hjust = 1))
  
  ggsave(paste0(output_path,"main_",statistic,".png"),
         width=9,
         height=6,
         units="in",
         dpi = 300)
}


## plot timeseries of R, MB, RMSE, COE for SO2 and CO -----------

df2 <- df_so2_co %>% 
  dplyr::filter(aggreg=="hourly") %>%
  dplyr::mutate(R2 = r^2) %>%
  dplyr::select(-n, -aggreg) %>%
  dplyr::mutate(pol=toupper(pol)) %>%
  pivot_wider(names_from = c("model"), values_from = statistic.list)


for(statistic in statistic.list){
  dodge <- 0.7
  p <- df2 %>%
    dplyr::mutate(year=as.factor(year)) %>%
    ggplot(aes(x=year, 
               ymin=!!sym(paste0(statistic,"_raw_model")), 
               ymax=!!sym(paste0(statistic,"_UK_model")), 
               color=pol, group=pol)) +
    # geom_linerange(position = position_dodge(width = dodge), size = 5)+
    geom_point(aes(y=!!sym(paste0(statistic,"_raw_model"))),shape=21,position=position_dodge(width=dodge))+
    geom_point(aes(y=!!sym(paste0(statistic,"_UK_model"))),shape=20,size=4,position=position_dodge(width=dodge))+
    geom_linerange(aes(x=year,y=!!sym(paste0(statistic,"_raw_model"))),
                   color="black", position = position_dodge(width = dodge))+
    labs(y=statistic,
         x="",
         color="Pollutant") +
    scale_color_viridis_d(alpha=1)+
    theme_classic()+
    theme(text = element_text(size=text_size),
          panel.grid.major = element_blank(), 
          panel.background = element_rect(colour = "black", size=0.5),
          axis.text.x = element_text(angle = 45, hjust = 1))
  
  ggsave(paste0(output_path,"so2_co_",statistic,".png"),
         width=9,
         height=6,
         units="in",
         dpi = 300)
}
