# plot obs vs model with limit

# inputs:
#   RDS or csv files with obs, raw, UK etc at each station 
#           (columns: date (with hour), ES0584A, ES...)

# outputs:
#   plots for each limit value


library(tidyr)
library(dplyr)
library(ggplot2)

args = commandArgs(TRUE)
if (length(args) == 2) {
  pollutant <- args[1]
  year <- args[2]
} else {
  cat("\n***Arguments missing***\n Arguments: pollutant year \n Pollutant codes: no2, o3, pm10, pm2p5, so2, co")
}


# PARAMETERS ----
input_path <- "/esarchive/scratch/ccarnere/RScripts/"
output_path <- "/esarchive/scratch/ccarnere/AQS_database/postprocessed/regional/statistics/"
text_size=15
cols <- c("#003F7D","orange","brown")
# pollutant <- "NO2"
# year <- 2022

# MAIN----

pollutant <- tolower(pollutant)
format <- ifelse(pollutant %in% c("so2","co"), "ghost", "gene")

# read input data ----
df_obs <- readRDS(paste0(input_path,"results_regional_",year,"/results_gene_",pollutant,"/output/df_obs_",format,"_",pollutant,".rds"))
df_raw <- readRDS(paste0(input_path,"results_regional_",year,"/results_gene_",pollutant,"/output/model_raw_at_AQMS_",format,"_",pollutant,"_df.rds"))
df_UK <- readRDS(paste0(input_path,"results_regional_",year,"/results_gene_",pollutant,"/output/LOOCV_",format,"_",pollutant,"_df.rds"))

# merge and transform data ----
df_obs <- df_obs %>% pivot_longer(cols=starts_with("ES"), names_to = "station")
df_raw <- df_raw %>% pivot_longer(cols=starts_with("ES"), names_to = "station")
df_UK <- df_UK %>% pivot_longer(cols=starts_with("ES"), names_to = "station")

df <- full_join(df_obs, df_raw, by=c("date","station"))
df <- full_join(df, df_UK, by=c("date","station"))
names(df) <- c("date","station","obs","raw","UK")
# df <- full_join(df_obs, df_UK, by=c("date","station"))
# names(df) <- c("date","station","obs","UK")


# legislated limit values ----
limits_directive <- data.frame("no2_mean"=40,
                               "no2_p99.79"=200,
                               "o3_aot"=18000,
                               "o3_8hmax"=120,
                               "o3_p93.2"=120,
                               "pm10_mean"=40,
                               "pm10_p90.4"=50,
                               "pm2p5_mean"=25)
vars <- names(limits_directive)
cond <- limits_directive %>% slice(1)

# plot ----
for(i in 1:length(vars)){
  
  if(grepl(tolower(pollutant), vars[i], fixed=T)){
    print(vars[i])
    
    p <- ggplot() +
      geom_point(data=df, aes(x=obs, y=raw, colour="c1")) +
      geom_point(data=df, aes(x=obs, y=UK, colour="c2")) +
      geom_hline(yintercept=cond[,i], linetype="dashed")+
      geom_vline(xintercept=cond[,i], linetype="dashed")+
      labs(y="model",
           x="observation") +
      scale_color_manual(name = "model",
                         breaks = c("c1", "c2"),
                         values = cols,
                         labels = c("raw", "UK"))+
      theme_classic()+
      coord_fixed()+
      ggtitle(vars[i])+
      theme(text = element_text(size=text_size),
            panel.grid.major = element_blank(), 
            panel.background = element_rect(colour = "black", size=0.5))

    # save plot----
    ggsave(file=paste0(output_path,"model_vs_obs_",vars[i],"_",year,".png"),
           plot = p,
           width=8,
           height=8,
           units="in",
           dpi = 300)
  }
}
